@@ -1,7 +1,7 @@
 #include "cs_rR.h"
 # define Dtol  1e-7
 
-cs *cs_rR(const cs *A, double nu, double nuR, const css *As, const cs *Roldinv, double Roldldet){
+cs *cs_rR(const cs *A, double nu, double nuR, const css *As, const cs *Roldinv, double Roldldet, const cs *pG){
     
 	cs *Rnew, *Rnewinv, *Ainv;
 	double Rnewldet, MH;
@@ -16,7 +16,8 @@ cs *cs_rR(const cs *A, double nu, double nuR, const css *As, const cs *Roldinv,
 	  for (j = 0 ; j < dimG; j++){
 		Rnewinv->i[cnt] = j;
 		Rnewinv->x[cnt] = 0.0;
-		cnt++;
+                A->x[i*dimG+j] -= pG->x[i*dimG+j];
+ 		cnt++;
 	  }
 	}
 	Rnewinv->p[dimG] = dimG*dimG;
@@ -28,22 +29,32 @@ cs *cs_rR(const cs *A, double nu, double nuR, const css *As, const cs *Roldinv,
 	cs_cov2cor(Rnew);
 		
 	Rnewldet = log(cs_invR(Rnew, Rnewinv));
-		
-	MH = Roldldet-Rnewldet;
 
+/*****************************************************/
+/*       From Eq A.4 in Liu and Daniels (2006)       */
+/*       using \pi_{1} = Eq 6 in Barnard (2000)      */
+/*  using \pi_{2} = Eq 3.4 in Liu and Daniels (2006) */
+/*****************************************************/
+
+        MH = Roldldet-Rnewldet;
+ 
 	for (i = 0 ; i < dimG; i++){
-		MH += log(Roldinv->x[i*dimG+i]);	
-		MH -= log(Rnewinv->x[i*dimG+i]);
+          MH += log(Roldinv->x[i*dimG+i]);
+          MH -= log(Rnewinv->x[i*dimG+i]);
 	}
+
 	MH *= 0.5*nuR;
 
 	if(MH<log(runif(0.0,1.0)) || Rnewldet<log(Dtol)){
-	  Rnewldet = cs_invR(Roldinv, Rnew);	// save old R
-	  for (i = 0 ; i < dimG; i++){
-	    Rnew->x[i*dimG+i] = 1.0;
+	  Rnewldet = cs_invR(Roldinv, Rnew);	// save old R	
+        }
+
+        for (i = 0 ; i < dimG; i++){
+          for (j = 0 ; j < dimG; j++){
+ 	    Rnew->x[i*dimG+j] *= sqrt((pG->x[i*dimG+i])*(pG->x[j*dimG+j]));
           }
-	}	
-	
+        }
+
         cs_spfree(Rnewinv);
         cs_spfree(Ainv);
 
